"use strict";
// ------------------------------------------------------------------------------
//  2017 RobotlegsJS. All Rights Reserved.
//
//  NOTICE: You are permitted to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
// ------------------------------------------------------------------------------
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@robotlegsjs/core");
var AsyncCommand_1 = require("./AsyncCommand");
var SubCommandInstanceMapping_1 = require("./SubCommandInstanceMapping");
var SubCommandMapping_1 = require("./SubCommandMapping");
var SubCommandMappingList_1 = require("./SubCommandMappingList");
var AbstractMacro = /** @class */ (function (_super) {
    __extends(AbstractMacro, _super);
    function AbstractMacro(context, injector) {
        var _this = _super.call(this, context) || this;
        _this._injector = injector.createChild();
        _this._mappings = new SubCommandMappingList_1.SubCommandMappingList();
        _this.prepare();
        return _this;
    }
    AbstractMacro.prototype.add = function (commandClass) {
        var mapping = new SubCommandMapping_1.SubCommandMapping(commandClass);
        this._mappings.addMapping(mapping);
        return mapping;
    };
    AbstractMacro.prototype.addInstance = function (command) {
        var mapping = new SubCommandInstanceMapping_1.SubCommandInstanceMapping(command);
        this._mappings.addMapping(mapping);
        return mapping;
    };
    AbstractMacro.prototype.remove = function (commandClass) {
        this._mappings.removeMappingsFor(commandClass);
    };
    AbstractMacro.prototype.executeCommand = function (mapping) {
        var command;
        var commandClass = mapping.commandClass;
        var payloads = mapping.payloads;
        var hasPayloads = payloads.length > 0;
        hasPayloads && this.mapPayloads(payloads);
        if (mapping.guards.length === 0 || core_1.guardsApprove(mapping.guards, this._injector)) {
            command = mapping.getOrCreateCommandInstance(this._injector);
            if (mapping.hooks.length > 0) {
                this._injector.bind(commandClass).toConstantValue(command);
                core_1.applyHooks(mapping.hooks, this._injector);
                this._injector.unbind(commandClass);
            }
        }
        hasPayloads && this.unmapPayloads(payloads);
        if (command && mapping.executeMethod) {
            var isAsync = command.constructor.prototype.registerCompleteCallback !== undefined;
            isAsync && command.registerCompleteCallback(this.commandCompleteHandler.bind(this));
            var executeMethod = command[mapping.executeMethod];
            executeMethod.apply(command);
            !isAsync && this.commandCompleteHandler(true);
        }
        else {
            this.commandCompleteHandler(true);
        }
    };
    AbstractMacro.prototype.mapPayloads = function (payloads) {
        for (var i = 0; i < payloads.length; i++) {
            var payload = payloads[i];
            this._injector.bind(payload.type).toConstantValue(payload.data);
        }
    };
    AbstractMacro.prototype.unmapPayloads = function (payloads) {
        for (var i = 0; i < payloads.length; i++) {
            var payload = payloads[i];
            this._injector.unbind(payload.type);
        }
    };
    AbstractMacro = __decorate([
        core_1.injectable(),
        __param(0, core_1.inject(core_1.IContext)),
        __param(1, core_1.inject(core_1.IInjector)),
        __metadata("design:paramtypes", [Object, Object])
    ], AbstractMacro);
    return AbstractMacro;
}(AsyncCommand_1.AsyncCommand));
exports.AbstractMacro = AbstractMacro;
//# sourceMappingURL=AbstractMacro.js.map