"use strict";
// ------------------------------------------------------------------------------
//  2017 RobotlegsJS. All Rights Reserved.
//
//  NOTICE: You are permitted to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
// ------------------------------------------------------------------------------
Object.defineProperty(exports, "__esModule", { value: true });
var SubCommandMappingList = /** @class */ (function () {
    function SubCommandMappingList() {
        this._mappings = [];
        this._mappingByCommand = new Map();
    }
    SubCommandMappingList.prototype.getList = function () {
        return this._mappings.concat();
    };
    SubCommandMappingList.prototype.addMapping = function (mapping) {
        this.storeMapping(mapping);
    };
    SubCommandMappingList.prototype.removeMapping = function (mapping) {
        this.deleteMapping(mapping);
    };
    SubCommandMappingList.prototype.removeMappingsFor = function (commandClass) {
        if (this._mappingByCommand.get(commandClass)) {
            var list = this._mappingByCommand.get(commandClass);
            var length_1 = list.length;
            while (length_1--) {
                this.deleteMapping(list[length_1]);
            }
            this._mappingByCommand.delete(commandClass);
        }
    };
    SubCommandMappingList.prototype.removeAllMappings = function () {
        if (this._mappings.length > 0) {
            var list = this._mappings.concat();
            var length_2 = list.length;
            while (length_2--) {
                this.deleteMapping(list[length_2]);
            }
        }
    };
    SubCommandMappingList.prototype.storeMapping = function (mapping) {
        var mappingByCommand = this._mappingByCommand.get(mapping.commandClass);
        if (!mappingByCommand) {
            mappingByCommand = new Array();
            this._mappingByCommand.set(mapping.commandClass, mappingByCommand);
        }
        mappingByCommand.push(mapping);
        this._mappings.push(mapping);
    };
    SubCommandMappingList.prototype.deleteMapping = function (mapping) {
        var mappingByCommand = this._mappingByCommand.get(mapping.commandClass);
        if (mappingByCommand) {
            mappingByCommand.splice(this._mappings.indexOf(mapping), 1);
            if (mappingByCommand.length === 0) {
                this._mappingByCommand.delete(mapping.commandClass);
            }
        }
        this._mappings.splice(this._mappings.indexOf(mapping), 1);
    };
    return SubCommandMappingList;
}());
exports.SubCommandMappingList = SubCommandMappingList;
//# sourceMappingURL=SubCommandMappingList.js.map